package me.eater.hefbrug.utils

import me.eater.hefbrug.logging.LogFormat
import org.apache.logging.log4j.util.StringBuilderFormattable


fun String.escape(): String =
    if (contains('"') || contains('\\') || contains('!') || contains('$') || contains('#')) {
        "\"${escapeChar('\\').escapeChar('"').escapeChar('$').escapeChar('#')}\""
    } else this


fun String.escapeChar(char: Char, escapeChar: Char = '\\'): String {
    return replace("$char", "$escapeChar$char")
}


fun String.handlebars(vararg items: Pair<String, Any?>): String = handlebars(items.toMap())

fun String.handlebars(map: Map<String, Any?>): String {
    return replace(Regex("""\{\{([a-z0-9_-]+)}}""")) { res ->
        when (val item = map[res.groupValues[1]]) {
            is LogFormat -> item.logFormat()
            is StringBuilderFormattable -> StringBuilder().also {
                item.formatTo(it)
                it.toString()
            }
            else -> "$item"
        }
    }
}